Struct comemo::Constraint
source · pub struct Constraint<T>where
T: Trackable + ?Sized,{ /* private fields */ }Expand description
Defines a constraint for a tracked method without arguments.
pub struct Constraint<T>where
T: Trackable + ?Sized,{ /* private fields */ }Defines a constraint for a tracked method without arguments.